#include <iomaxq200x.h>
#include <intrinsics.h>
/*
 * Instructions for running this demo:
 *
 *       SW6.2, SW6.5, SW6.8 : Enable push button interrupts on P5.2 and P7.1
 */

/* 
 * Compatible with the MAXQ2000 EVKit Rev. B
 */
#define LCD0_PATTERN_C    0x039
#define LCD0_PATTERN_F    0x071

#define LCD_PATTERN_0     0x03F
#define LCD_PATTERN_1     0x006
#define LCD_PATTERN_2     0x05B
#define LCD_PATTERN_3     0x04F
#define LCD_PATTERN_4     0x066
#define LCD_PATTERN_5     0x06D
#define LCD_PATTERN_6     0x07D
#define LCD_PATTERN_7     0x007
#define LCD_PATTERN_8     0x07F
#define LCD_PATTERN_9     0x067

int PATTERNS[] = { LCD_PATTERN_0, LCD_PATTERN_1, LCD_PATTERN_2, LCD_PATTERN_3, LCD_PATTERN_4, 
                   LCD_PATTERN_5, LCD_PATTERN_6, LCD_PATTERN_7, LCD_PATTERN_8, LCD_PATTERN_9 };

#define DISPLAY_MODE_CLOCK    1
#define DISPLAY_MODE_COUNTER  2

int pbcount;
int rtccount;
int display_mode;

/******************************************************************************
 * Lights up the solitary 'one' segment on our LCD panel.
 */
show100s()
{
  LCD4 = 0x40;
}

/******************************************************************************
 * Returns the value that will need to be placed in one of LCD0-LCD3 to display 
 * a digit 0-9.  No bounds checking is done here.  If you ask for a digit other
 * than 0-9, you will get a bogus display.
 */
int getLCDDigit(int digit)
{
    return PATTERNS[digit];
}

int show(int value)
{
   if (value >= 10000)
      LCD4 = 0x40;
   else
      LCD4 = 0;
   LCD3 = getLCDDigit((value / 1000) % 10);
   LCD2 = getLCDDigit((value / 100) % 10);
   LCD1 = getLCDDigit((value / 10) % 10);
   LCD0 = getLCDDigit((value) % 10);
   return 0;
}

void initLCD()
{
  //LCRA_bit.FRM = 7;   // Set up frame frequency.
  //LCRA_bit.LCCS = 1;  // Set clock source to HF/128.
  //LCRA_bit.DUTY = 0;  // Set up static duty cycle.
  //LCRA_bit.LRA = 0;   // Set R-adj to 0.
  //LCRA_bit.LRIGC = 1; // Select external LCD drive power.
  LCRA = 0x03E0;        // Just do them all at once...

  LCFG_bit.PCF = 0x0F;// Set up all segments as outputs.
  LCFG_bit.OPM = 1;   // Set to normal operation mode.
  LCFG_bit.DPE = 1;   // Enable display.
}

/******************************************************************************
 * Setup the external interrupts for EI 10 and 15 (pins P5.2 and P7.1).
 */
void initPushButtons()
{
    EIE1 = 0x84;                          // enable interrupts 10 and 15
    EIES1 = 0x84;                         // set up edge transitions
    IMR |= 0x02;                          // enable interrupts for module 1
}

void initRTC()
{
    RCNT = 0x0000;                        // turn on the 32kHz
    while (RCNT & 0x08) {}
    RCNT = 0x8000;                        // write enable the real-time clock enable bit
    RCNT = 0x8001;                        // start the real time clock
    RCNT = 0x0001;                        // turn off RTCE write access 
    IMR |= 0x01;                          // enable interrupts for module 0
}

void updateTimeAlarm()
{
    int temp = RTSL;
    RCNT = RCNT & 0xFFFD;                // disable the alarm time of day enable
    temp = temp + 1;
    while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
    RASL = temp;
    if ((temp & 0xffff)==0)
    {
        // there was a rollover
        RASH = RTSH + 1;
    }
    else
        RASH = RTSH;
    //while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
    RCNT = RCNT | 0x02;                  // turn the alarm time of day back on
}

void showTime()
{
    int minutes, hours;
    unsigned long seconds = RTSH;
    seconds = seconds << 16;
    seconds = seconds | RTSL;
    minutes = seconds / 60;
    hours = minutes / 60;
    hours = hours % 24;
    minutes = minutes % 60;
    show((hours * 100)+minutes);
    rtccount++;
    if (rtccount & 1)
        LCD4 = LCD4 | 0x10;                    // toggle the middle colon
}

#pragma vector = 0
__interrupt void realTimeClockInterrupt()
{
  
  // check for interrupt 10 (0x04) or interrupt 15 (0x80)
  if ((RCNT & 0x40) == 0x40)
  {
     RCNT = RCNT & 0xFFBF;                // turn off the RTC alarm
     showTime();                          // show the first time
     updateTimeAlarm();
  }
}




/******************************************************************************
 * The interrupt handler for module 1, including my external interrupts.
 * Must be careful in here to make sure I react only to the interrupts I 
 * want, since some pins are dual purpose.
 */
#pragma vector = 1
__interrupt void pushButtonInterrupt()
{
  int temp;
  unsigned long time;
  temp = PI7;  
  pbcount++;                              // increment our debug variable 'pushbuttoncount'
  // check for interrupt 10 (0x04) or interrupt 15 (0x80)
  if (EIF1 & 0x04)
  {
      // interrupt 10
      if (display_mode == DISPLAY_MODE_CLOCK)
      {
          // first check to see if the other is held down.
          // if so, increment the hours.
          if ((temp & 0x02) == 0)
          {
              // then the other button is being held down.
              while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
              RCNT = 0x8001;                       // set write enable bit
              while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
              RCNT = 0x8000;                       // set RTCE to clock NOT enabled
              time = RTSH;
              time = time << 16;
              time = time | RTSL;
              time += (60 * 60);                   // add 1 hours worth of seconds
              while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
              RTSH = (time >> 16) & 0x0ffff;
              while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
              RTSL = time & 0x0ffff;
              while (RCNT & 0x08) {}               // wait for the busy flag to indicate not busy
              RCNT = 0x8001;                       // turn the clock back on
              showTime();                          // show the first time
              updateTimeAlarm();
              RCNT = 0x0003;
          }
          else
          {
              display_mode = DISPLAY_MODE_COUNTER;
              RCNT = 0x0001;                       // turn off the RTC alarm
          }
      }
      else
      {
          display_mode = DISPLAY_MODE_CLOCK;
          showTime();                          // show the first time
          updateTimeAlarm();
          RCNT = 0x0003;                       // turn on the RTC alarm
      }
  }
  if (EIF1 & 0x80)
  {
      // interrupt 15
      // let's add a minute to the time
      while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
      RCNT = 0x8001;                       // set write enable bit
      while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
      RCNT = 0x8000;                       // set RTCE to clock NOT enabled
      time = RTSH;
      time = time << 16;
      time = time | RTSL;
      time += (60);                        // add 1 minute worth of seconds
      while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
      RTSH = (time >> 16) & 0x0ffff;
      while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
      RTSL = time & 0x0ffff;
      while (RCNT & 0x08) {}               // wat for the busy flag to indicate not busy
      RCNT = 0x8001;                       // turn the clock back on
      time++;
      showTime();                          // show the first time
      updateTimeAlarm();
      RCNT = 0x0003;
  }
  
  EIF1 = 0;                               // clear the interrupt condition
}


/******************************************************************************
 * Main application entry point.  Initialize the serial port, timer, SPI/1407,
 * LCD, and external interrupts.  Then just do nothing.  The interrupts will
 * take care of everything else.
 */
void main()
{
  int counter = 0;
  int i=0;
  
  pbcount = 0;
  rtccount = 0;
  display_mode = DISPLAY_MODE_COUNTER;
  
  initPushButtons();
  __enable_interrupt();
  initLCD();
  initRTC();
  
  while(1)
  {
      for (i=0;i<32000;i++)
      {
         // just a delay loop
      }
      
      if (display_mode == DISPLAY_MODE_COUNTER)
          show(counter);
      if (counter>=19999)
          counter = 0;
      counter++;
  }
}

